#include <fstream>
#include <string.h>
#include "optFileWriter.h"

/** Export header for the output file */
static const char *header = 
    "<?xml version=\"1.0\" standalone=\"yes\"?>\n<graphopt>\n";

/** Export footer for the output file */
static const char *footer =
    "</graphopt>\n";


/** Constructor for the Graphopt file writer
 */
optFileWriter::optFileWriter(char *what_file, nodes *what_nodes)
    : filename(what_file), my_nodes(what_nodes) {
}


/** Write the node data to a file
 */
void optFileWriter::write() {
    std::ofstream outfile;

    outfile.open(filename.c_str());
    outfile << header;
    write_nodes(outfile);
    outfile << footer;
    outfile.close();
}


/** Write the body of the output data to the file.
 *
 * The header should be emitted beforehand, and the footer afterwards.
 */
void optFileWriter::write_nodes(std::ofstream &outfile)
{
    /* Assign each node a unique identifier for export */
    for (int i = 0; i < my_nodes->number_of_nodes(); i++)
    {
        char temp[40];
        snprintf(temp, 40, "%d", i);
        temp[39] = '\0';

        my_nodes->get_node(i)->set_export_id(temp);
    }

    for (int temp = 0; temp < my_nodes->number_of_nodes(); temp++) {
        node *temp_node = my_nodes->get_node(temp);

        outfile << "  <node>" << std::endl;
        outfile << "    <id>" << temp_node->get_export_id() << "</id>" << std::endl;
        outfile << "    <name>" << temp_node->get_name() << "</name>" << std::endl;
        outfile << "    <x_pos>" << temp_node->x_pos << "</x_pos>" << std::endl;
        outfile << "    <y_pos>" << temp_node->y_pos << "</y_pos>" << std::endl;
        outfile << "    <links>" << std::endl;

        std::vector<node *> links = temp_node->get_connections();

        for (int i = 0; i < links.size(); i++) {
            outfile << "      <link>" << links[i]->get_export_id() << "</link>" << std::endl;
        }
        
        outfile << "    </links>" << std::endl;
        outfile << "  </node>" << std::endl;
    }
}

